# SPDX-FileCopyrightText: 2019 Dan Halbert for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_bluefruit_connect`
============================

This module helps you to communicate with the `Adafruit Bluefruit Connect App
<https://learn.adafruit.com/bluefruit-le-connect>`_ or use its protocols with
boards that run `Adafruit CircuitPython firmware
<https://circuitpython.org/downloads>`_.

* Author(s): Dan Halbert for Adafruit Industries

"""

# imports

__version__ = "0.0.0-auto.0"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_BluefruitConnect.git"
